#! /usr/bin/python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

from buildbot.process import step
from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, SKIPPED

class FindPkgConfigPath(step.ShellCommand):
    name = "pkg-config"
    description = ["finding",  "pkg-config", "files"]
    descriptionDone = ["pkg-config", "files"]
    #command = ["cd",  "../..", "&&", "pwd"]
    command = ['sh', '-c', '"cd ../.. && pwd"']
    #command = ["cd .. && pwd"]

class IncMake(step.ShellCommand):
    name = "incmake"
    description = ["running",  "incr", "make"]
    descriptionDone = ["incr", "make"]
    command = ["make"]

class Autogen(step.ShellCommand):
    """
    Run ./autogen.sh with optional "options" passed in
    """
    name = "autogen"
    #warnOnFailure = 0
    haltOnFailure = 1
    description = ["autogenning"]
    descriptionDone = ["autogen"]
    command = ["./autogen.sh"]

    def __init__(self, **kwargs):
        """
        @param options: options to pass to autogen
        """
        options = ""
        if "options" in kwargs.keys():
            options = kwargs['options']
            del kwargs['options']
            kwargs['command'] = self.command[:]
            kwargs['command'].extend(options)

        step.ShellCommand.__init__(self, **kwargs)

class AutogenAfterIncMake(Autogen):
    def start(self, errorMessage=None):
       previousResult = self.build.results[-1]
       # if the inc make succeeded, we don't need to autogen
       if previousResult == SUCCESS:
           return SKIPPED
       # otherwise, do what we would normally do
       return step.ShellCommand.start(self, errorMessage)

class CompileAfterIncMake(step.ShellCommand):
    name = "compile"
    #warnOnFailure = 0
    haltOnFailure = 1
    description = ["compiling"]
    descriptionDone = ["compile"]
    command = ["make"]

    def start(self, errorMessage=None):
       previousResult = self.build.results[-1]
       # if the autogen was skipped, we don't need to compile, since inc make
       # worked
       if previousResult == SKIPPED:
           return SKIPPED
       # otherwise, do what we would normally do
       return step.ShellCommand.start(self, errorMessage)

class Make(step.ShellCommand):
    def __init__(self, **kwargs):
        target = "all"
        if "target" in kwargs.keys():
            target = kwargs['target']
            del kwargs['target']

        # set defaults if not given for some vars
        for key, value in [
            ('name',            target),
            ('description',     ["running", "make", target]),
            ('descriptionDone', ["make", target]),
            ('command',         ["make", target])]:
            if not key in kwargs.keys():
                kwargs[key] = value

        step.ShellCommand.__init__(self, **kwargs)

class RpmBuild(step.ShellCommand):
    """
    Build an rpm by removing tarball, running "make dist" and running rpmbuild.
    """
    name = "rpmbuild"
    description = ["rpm",  "building"]
    descriptionDone = ["rpm", "build"]

    def __init__(self, **kwargs):
        """
        @param tarball:  tarball to build from
        @param buildarg: rpmbuild build arg to pass (defaults to -tb)
        """

        tarball = "*.tar.gz"
        if "tarball" in kwargs.keys():
            tarball = kwargs['tarball']
            del kwargs['tarball']

        buildarg = "-tb"
        if "buildarg" in kwargs.keys():
            buildarg = kwargs['buildarg']
            del kwargs['buildarg']

        command = "rm -rf rpm && "
        for d in ['RPMS', 'SRPMS', 'SOURCES', 'SPECS', 'BUILD']:
            command += ('mkdir -p rpm/%s && ' % d)

        command += ('rm -f %s && ' % tarball)
        command += ('make dist && ')
        command += ('rpmbuild --define "_topdir `pwd`/rpm" %s %s ' % (
            buildarg, tarball))

        # FIXME: chmod because twistd/buildbot runs with umask 077
        command += ('&& chmod -R ga+r rpm ')

        kwargs['command'] = command

        step.ShellCommand.__init__(self, **kwargs)

class RpmUpload(step.ShellCommand):
    """
    Upload the built RPMS to some location using rsync.
    """
    name = "upload"
    description = ["rpm",  "uploading"]
    descriptionDone = ["rpm", "uploaded"]

    def __init__(self, **kwargs):
        """
        @param destination: destination to rsync to
        """

        if "destination" not in kwargs.keys():
            raise KeyError("Need a destination to upload to")
        destination = kwargs['destination']

        command = "rsync `find rpm -name '*.rpm'` %s" % destination

        kwargs['command'] = command

        step.ShellCommand.__init__(self, **kwargs)

class TarballUpload(step.ShellCommand):
    """
    Upload the tarball to some location using rsync.
    """
    name = "upload"
    description = ["tarball",  "uploading"]
    descriptionDone = ["tarball", "uploaded"]

    def __init__(self, **kwargs):
        """
        @param destination: destination to rsync to
        @param tarball:     tarball to rsync
        """
        tarball = "*.tar.gz"
        if "tarball" in kwargs.keys():
            tarball = kwargs['tarball']
            del kwargs['tarball']

        if "destination" not in kwargs.keys():
            raise KeyError("Need a destination to upload to")
        destination = kwargs['destination']

        # FIXME: chmod because twistd/buildbot runs with umask 077
        command = 'chmod ga+r %s' % tarball
        command += " && rsync %s %s" % (tarball, destination)

        kwargs['command'] = command

        step.ShellCommand.__init__(self, **kwargs)

class MachRpmBuild(step.ShellCommand):
    """
    Build an rpm by running mach on the spec.
    """
    name = "machrpmbuild"
    description = ["mach",  "building"]
    descriptionDone = ["mach", "build"]

    def __init__(self, **kwargs):
        """
        @param spec:     spec to build from
        @param root:     the mach root to build in
        """

        spec = "*.spec"
        if "spec" in kwargs.keys():
            spec = kwargs['spec']
            del kwargs['spec']

        rootarg = None
        if "root" in kwargs.keys():
            rootarg = "-r %s" % kwargs['root']
            del kwargs['root']

        command = "mach %s build %s" % (rootarg, spec)

        kwargs['command'] = command

        step.ShellCommand.__init__(self, **kwargs)
